// This plugin makes API requests at specific points during Cypress test execution
// - Before any test in the entire test suite runs
// - Before each test file runs
// - After each test file completes

/**
 * Makes an API request with the provided configuration
 * @param {Object} options - Request options
 * @param {string} options.url - The URL to request
 * @param {string} options.method - HTTP method (GET, POST, etc.)
 * @param {Object} [options.body] - Request body for POST/PUT/PATCH requests
 * @param {Object} [options.headers] - Request headers
 * @param {string} eventType - Description of when this request is being made
 * @returns {Promise} - Promise resolving to the API response
 */
const makeApiRequest = async (options, eventType) => {
  console.log(`Making API request for event: ${eventType}`);

  try {
    const response = await fetch(options.url, {
      method: options.method || "GET",
      headers: {
        "Content-Type": "application/json",
        ...options.headers,
      },
      ...(options.body && { body: JSON.stringify(options.body) }),
    });

    const data = await response.json();
    console.log(
      `API request for ${eventType} completed with status: ${response.status}`,
    );
    console.log(data);
    return data;
  } catch (error) {
    console.error(`API request for ${eventType} failed:`, error);
    throw error;
  }
};

/**
 * Configuration for the different API requests to be made
 */
const apiRequests = {
  beforeAllTests: {},
  beforeTestFile: {},
  afterTestFile: {},
};

/**
 * Sets up the plugin in Cypress
 * @param {Object} on - Cypress event registration function
 * @param {Object} config - Cypress configuration
 */
// Track if event handlers have been registered to prevent duplicate registrations
let eventHandlersRegistered = false;

function setupApiRequestPlugin(on, config) {
  // If event handlers are already registered, don't register them again
  if (eventHandlersRegistered) {
    console.log("API plugin event handlers already registered, skipping...");
    return;
  }

  // Track whether we've already run the before all tests request
  let beforeAllTestsExecuted = false;
  // Track the current run ID across all tests
  let currentRunId = null;

  // Before all tests run (only once)
  on("before:run", async () => {
    if (!beforeAllTestsExecuted) {
      apiRequests.beforeAllTests = {
        url: `${config.historyTrackerURL || ""}/api/run/create`,
        method: "POST",
        body: {
          runName: `Cypress Run @ ${new Date().toLocaleTimeString()}`,
        },
      };

      const res = await makeApiRequest(
        apiRequests.beforeAllTests,
        "beforeAllTests",
      );
      currentRunId = res.runId;

      beforeAllTestsExecuted = true;
    }
  });

  // TODO: FIGURE OUT WHY THIS RUNS TWICE

  // Process individual test events
  on("before:spec", async (test) => {
    if (!currentRunId) return;

    const testRequest = {
      url: `${config.historyTrackerURL || ""}/api/run/create`,
      method: "POST",
      body: {
        runId: currentRunId,
        testName: test.name,
        path: test.relative,
      },
    };

    await makeApiRequest(testRequest, "beforeTest");
  });

  on("after:spec", async (test, results) => {
    if (!currentRunId) return;

    const testRequest = {
      url: `${config.historyTrackerURL || ""}/api/run/create`,
      method: "POST",
      body: {
        runId: currentRunId,
        testName: test.name,
        path: test.relative,
        runType: "end",
        success: results.stats.failures === 0,
      },
    };

    console.log("specs", results);

    await makeApiRequest(testRequest, "afterTest");
  });

  // Mark event handlers as registered to prevent duplicate registrations
  eventHandlersRegistered = true;
  console.log("API plugin event handlers registered successfully");
}

module.exports = setupApiRequestPlugin;
